@@ -5,13 +5,18 @@ import tim.prune.correlate.PhotoCorrelator;
 import tim.prune.function.*;
 import tim.prune.function.charts.Charter;
 import tim.prune.function.compress.CompressTrackFunction;
+import tim.prune.function.compress.MarkPointsInRectangleFunction;
 import tim.prune.function.distance.DistanceFunction;
 import tim.prune.function.edit.PointNameEditor;
 import tim.prune.function.estimate.EstimateTime;
 import tim.prune.function.estimate.LearnParameters;
 import tim.prune.function.gpsies.GetGpsiesFunction;
 import tim.prune.function.gpsies.UploadGpsiesFunction;
+import tim.prune.function.sew.SewTrackSegmentsFunction;
+import tim.prune.function.sew.SplitSegmentsFunction;
+import tim.prune.function.srtm.DownloadSrtmFunction;
 import tim.prune.function.srtm.LookupSrtmFunction;
+import tim.prune.function.weather.GetWeatherForecastFunction;
 import tim.prune.load.AudioLoader;
 import tim.prune.load.BabelLoadFromFile;
 import tim.prune.load.BabelLoadFromGps;
@@ -38,12 +43,16 @@ public abstract class FunctionLibrary
 	public static GenericFunction FUNCTION_SAVECONFIG  = null;
 	public static GenericFunction FUNCTION_EDIT_WAYPOINT_NAME = null;
 	public static RearrangeWaypointsFunction FUNCTION_REARRANGE_WAYPOINTS = null;
+	public static GenericFunction FUNCTION_SPLIT_SEGMENTS = null;
+	public static GenericFunction FUNCTION_SEW_SEGMENTS = null;
 	public static GenericFunction FUNCTION_REARRANGE_PHOTOS = null;
 	public static GenericFunction FUNCTION_COMPRESS = null;
 	public static GenericFunction FUNCTION_DELETE_RANGE = null;
 	public static GenericFunction FUNCTION_CROP_TRACK = null;
+	public static GenericFunction FUNCTION_MARK_IN_RECTANGLE = null;
 	public static GenericFunction FUNCTION_INTERPOLATE = null;
 	public static GenericFunction FUNCTION_LOOKUP_SRTM = null;
+	public static GenericFunction FUNCTION_DOWNLOAD_SRTM = null;
 	public static GenericFunction FUNCTION_LOOKUP_WIKIPEDIA = null;
 	public static GenericFunction FUNCTION_SEARCH_WIKIPEDIA = null;
 	public static GenericFunction FUNCTION_DOWNLOAD_OSM = null;
@@ -71,6 +80,7 @@ public abstract class FunctionLibrary
 	public static GenericFunction FUNCTION_LEARN_ESTIMATION_PARAMS = null;
 	public static GenericFunction FUNCTION_GET_GPSIES = null;
 	public static GenericFunction FUNCTION_UPLOAD_GPSIES = null;
+	public static GenericFunction FUNCTION_GET_WEATHER_FORECAST = null;
 	public static GenericFunction FUNCTION_LOAD_AUDIO = null;
 	public static GenericFunction FUNCTION_REMOVE_AUDIO = null;
 	public static GenericFunction FUNCTION_CORRELATE_AUDIOS = null;
@@ -105,12 +115,16 @@ public abstract class FunctionLibrary
 		FUNCTION_SAVECONFIG = new SaveConfig(inApp);
 		FUNCTION_EDIT_WAYPOINT_NAME = new PointNameEditor(inApp);
 		FUNCTION_REARRANGE_WAYPOINTS = new RearrangeWaypointsFunction(inApp);
+		FUNCTION_SPLIT_SEGMENTS = new SplitSegmentsFunction(inApp);
+		FUNCTION_SEW_SEGMENTS = new SewTrackSegmentsFunction(inApp);
 		FUNCTION_REARRANGE_PHOTOS = new RearrangePhotosFunction(inApp);
 		FUNCTION_COMPRESS = new CompressTrackFunction(inApp);
 		FUNCTION_DELETE_RANGE = new DeleteSelectedRangeFunction(inApp);
 		FUNCTION_CROP_TRACK = new CropToSelection(inApp);
+		FUNCTION_MARK_IN_RECTANGLE = new MarkPointsInRectangleFunction(inApp);
 		FUNCTION_INTERPOLATE = new InterpolateFunction(inApp);
 		FUNCTION_LOOKUP_SRTM = new LookupSrtmFunction(inApp);
+		FUNCTION_DOWNLOAD_SRTM = new DownloadSrtmFunction(inApp);
 		FUNCTION_LOOKUP_WIKIPEDIA = new GetWikipediaFunction(inApp);
 		FUNCTION_SEARCH_WIKIPEDIA = new SearchWikipediaNames(inApp);
 		FUNCTION_DOWNLOAD_OSM = new DownloadOsmFunction(inApp);
@@ -137,6 +151,7 @@ public abstract class FunctionLibrary
 		FUNCTION_LEARN_ESTIMATION_PARAMS = new LearnParameters(inApp);
 		FUNCTION_GET_GPSIES = new GetGpsiesFunction(inApp);
 		FUNCTION_UPLOAD_GPSIES = new UploadGpsiesFunction(inApp);
+		FUNCTION_GET_WEATHER_FORECAST = new GetWeatherForecastFunction(inApp);
 		FUNCTION_LOAD_AUDIO = new AudioLoader(inApp);
 		FUNCTION_REMOVE_AUDIO = new RemoveAudioFunction(inApp);
 		FUNCTION_CORRELATE_AUDIOS = new AudioCorrelator(inApp);
